@@ -4,15 +4,9 @@ import logging
 import time
 from typing import Callable, Coroutine, Dict, List, Optional, Tuple
 
-from flux_led.protocol import (
-    ProtocolLEDENET8Byte,
-    ProtocolLEDENETAddressableA3,
-    ProtocolLEDENETOriginal,
-)
-
 from .aioprotocol import AIOLEDENETProtocol
 from .aioscanner import AIOBulbScanner
-from .base_device import PROTOCOL_PROBES, LEDENETDevice
+from .base_device import PROTOCOL_PROBES, DeviceType, LEDENETDevice
 from .const import (
     COLOR_MODE_CCT,
     COLOR_MODE_DIM,
@@ -28,10 +22,20 @@ from .const import (
     STATE_WARM_WHITE,
     MultiColorEffects,
 )
+from .protocol import (
+    POWER_RESTORE_BYTES_TO_POWER_RESTORE,
+    PowerRestoreState,
+    PowerRestoreStates,
+    ProtocolLEDENET8Byte,
+    ProtocolLEDENETAddressableA3,
+    ProtocolLEDENETAddressableChristmas,
+    ProtocolLEDENETOriginal,
+)
 from .utils import color_temp_to_white_levels, rgbw_brightness, rgbww_brightness
 
 _LOGGER = logging.getLogger(__name__)
 
+
 COMMAND_SPACING_DELAY = 1
 MAX_UPDATES_WITHOUT_RESPONSE = 4
 POWER_STATE_TIMEOUT = 1.2  # number of seconds before declaring on/off failed
@@ -58,6 +62,7 @@ class AIOWifiLedBulb(LEDENETDevice):
         super().__init__(ipaddr, port, timeout)
         self._lock = asyncio.Lock()
         self._aio_protocol: Optional[AIOLEDENETProtocol] = None
+        self._power_restore_future: "asyncio.Future[bool]" = asyncio.Future()
         self._ic_future: "asyncio.Future[bool]" = asyncio.Future()
         self._on_futures: List["asyncio.Future[bool]"] = []
         self._off_futures: List["asyncio.Future[bool]"] = []
@@ -67,16 +72,43 @@ class AIOWifiLedBulb(LEDENETDevice):
         self._pixels_per_segment: Optional[int] = None
         self._segments: Optional[int] = None
         self._last_update_time: float = NEVER_TIME
+        self._power_restore_state: Optional[PowerRestoreStates] = None
         self._buffer = b""
         self.loop = asyncio.get_running_loop()
 
+    @property
+    def power_restore_states(self) -> Optional[PowerRestoreStates]:
+        """Returns the power restore states for all channels."""
+        return self._power_restore_state
+
     async def async_setup(self, updated_callback: Callable[[], None]) -> None:
         """Setup the connection and fetch initial state."""
         self._updated_callback = updated_callback
         await self._async_determine_protocol()
         assert self._protocol is not None
-        if not self._protocol.zones:
+        if self._protocol.zones:
+            await self._async_addressable_setup()
             return
+        if self.device_type == DeviceType.Switch:
+            await self._async_switch_setup()
+
+    async def _async_switch_setup(self) -> None:
+        """Setup a switch."""
+        assert self._protocol is not None
+        await self._async_send_msg(self._protocol.construct_power_restore_state_query())
+        try:
+            await asyncio.wait_for(self._power_restore_future, timeout=self.timeout)
+        except asyncio.TimeoutError:
+            self.set_unavailable()
+            raise RuntimeError("Could not determine power restore state")
+
+    async def _async_addressable_setup(self) -> None:
+        """Setup an addressable light."""
+        assert self._protocol is not None
+        if isinstance(self._protocol, ProtocolLEDENETAddressableChristmas):
+            self._pixels_per_segment = 6  # currently hard coded
+            return
+
         assert isinstance(self._protocol, ProtocolLEDENETAddressableA3)
         await self._async_send_msg(self._protocol.construct_request_strip_setting())
         try:
@@ -290,15 +322,18 @@ class AIOWifiLedBulb(LEDENETDevice):
     async def async_set_zones(
         self,
         rgb_list: List[Tuple[int, int, int]],
-        speed: int,
-        effect: MultiColorEffects,
+        speed: int = 100,
+        effect: MultiColorEffects = MultiColorEffects.STATIC,
     ) -> None:
         """Set zones."""
         assert self._protocol is not None
         if not self._protocol.zones:
             raise ValueError("{self.model} does not support zones")
         assert self._pixels_per_segment is not None
-        assert isinstance(self._protocol, ProtocolLEDENETAddressableA3)
+        assert isinstance(
+            self._protocol,
+            (ProtocolLEDENETAddressableA3, ProtocolLEDENETAddressableChristmas),
+        )
         await self._async_send_msg(
             self._protocol.construct_zone_change(
                 self._pixels_per_segment, rgb_list, speed, effect
@@ -373,6 +408,28 @@ class AIOWifiLedBulb(LEDENETDevice):
         await AIOBulbScanner().async_disable_remote_access(self.ipaddr)
         self._async_stop()
 
+    async def async_set_power_restore(
+        self,
+        channel1: Optional[PowerRestoreState] = None,
+        channel2: Optional[PowerRestoreState] = None,
+        channel3: Optional[PowerRestoreState] = None,
+        channel4: Optional[PowerRestoreState] = None,
+    ) -> None:
+        new_power_restore_state = self._power_restore_state
+        assert new_power_restore_state is not None
+        if channel1 is not None:
+            new_power_restore_state.channel1 = channel1
+        if channel2 is not None:
+            new_power_restore_state.channel2 = channel2
+        if channel3 is not None:
+            new_power_restore_state.channel3 = channel3
+        if channel4 is not None:
+            new_power_restore_state.channel4 = channel4
+        assert self._protocol is not None
+        await self._async_send_msg(
+            self._protocol.construct_power_restore_state_change(new_power_restore_state)
+        )
+
     async def _async_connect(self) -> None:
         """Create connection."""
         _, self._aio_protocol = await asyncio.wait_for(
@@ -430,30 +487,55 @@ class AIOWifiLedBulb(LEDENETDevice):
         """Process a full message (maybe reassembled)."""
         assert self._protocol is not None
         self.set_available()
-        assert self._updated_callback is not None
         prev_state = self.raw_state
-        if self._protocol.is_valid_addressable_response(msg):
-            self.process_addressable_response(msg)
-        elif self._protocol.is_valid_state_response(msg):
+        if self._protocol.is_valid_outer_message(msg):
+            msg = self._protocol.extract_inner_message(msg)
+
+        if self._protocol.is_valid_state_response(msg):
             self._async_process_state_response(msg)
         elif self._protocol.is_valid_power_state_response(msg):
             self.process_power_state_response(msg)
         elif self._protocol.is_valid_ic_response(msg):
             self.process_ic_response(msg)
+        elif self._protocol.is_valid_power_restore_state_response(msg):
+            self.process_power_restore_state_response(msg)
         else:
             return
         if self.raw_state == prev_state:
             return
+        self._process_futures_and_callbacks()
+
+    def _process_futures_and_callbacks(self) -> None:
+        """Called when state changes."""
         futures = self._on_futures if self.is_on else self._off_futures
         for future in futures:
             if not future.done():
                 future.set_result(True)
         futures.clear()
+        assert self._updated_callback is not None
         try:
             self._updated_callback()
         except Exception as ex:  # pylint: disable=broad-except
             _LOGGER.error("Error while calling callback: %s", ex)
 
+    def process_power_restore_state_response(self, msg: bytes) -> None:
+        """Process a power restore state response.
+        Power on state always off
+        f0 32 ff f0 f0 f0 f1
+        Power on state always on
+        f0 32 0f f0 f0 f0 01
+        Power on state keep last state
+        f0 32 f0 f0 f0 f0 e2
+        """
+        self._power_restore_state = PowerRestoreStates(
+            channel1=POWER_RESTORE_BYTES_TO_POWER_RESTORE.get(msg[2]),
+            channel2=POWER_RESTORE_BYTES_TO_POWER_RESTORE.get(msg[3]),
+            channel3=POWER_RESTORE_BYTES_TO_POWER_RESTORE.get(msg[4]),
+            channel4=POWER_RESTORE_BYTES_TO_POWER_RESTORE.get(msg[5]),
+        )
+        if not self._power_restore_future.done():
+            self._power_restore_future.set_result(True)
+
     def process_ic_response(self, msg: bytes) -> bool:
         assert self._aio_protocol is not None
         high_byte = msg[2]
@@ -475,16 +557,6 @@ class AIOWifiLedBulb(LEDENETDevice):
             self._ic_future.set_result(True)
         return True
 
-    def process_addressable_response(self, msg: bytes) -> bool:
-        assert self._aio_protocol is not None
-        _LOGGER.debug(
-            "%s <= Extracted response (%s) (%d)",
-            self._aio_protocol.peername,
-            " ".join(f"0x{x:02X}" for x in msg[10:-1]),
-            len(msg[10:-1]),
-        )
-        return self._async_process_state_response(msg[10:-1])
-
     async def _async_send_msg(self, msg: bytearray) -> None:
         """Write a message on the socket."""
         if not self._aio_protocol:
