@@ -1,6 +1,9 @@
 """FluxLED Protocols."""
 
 from abc import abstractmethod
+import colorsys
+from dataclasses import dataclass
+from enum import Enum
 import logging
 from typing import List, NamedTuple, Optional, Tuple, Union
 
@@ -13,6 +16,21 @@ from .const import (
 )
 from .utils import utils, white_levels_to_scaled_color_temp
 
+
+class PowerRestoreState(Enum):
+    ALWAYS_OFF = 0xFF
+    ALWAYS_ON = 0x0F
+    LAST_STATE = 0xF0
+
+
+@dataclass
+class PowerRestoreStates:
+    channel1: Optional[PowerRestoreState]
+    channel2: Optional[PowerRestoreState]
+    channel3: Optional[PowerRestoreState]
+    channel4: Optional[PowerRestoreState]
+
+
 _LOGGER = logging.getLogger(__name__)
 
 # Protocol names
@@ -27,6 +45,7 @@ PROTOCOL_LEDENET_ADDRESSABLE_A1 = "LEDENET_ADDRESSABLE_A1"
 PROTOCOL_LEDENET_ADDRESSABLE_A2 = "LEDENET_ADDRESSABLE_A2"
 PROTOCOL_LEDENET_ADDRESSABLE_A3 = "LEDENET_ADDRESSABLE_A3"
 PROTOCOL_LEDENET_CCT = "LEDENET_CCT"
+PROTOCOL_LEDENET_ADDRESSABLE_CHRISTMAS = "LEDENET_CHRISTMAS"
 
 TRANSITION_BYTES = {
     TRANSITION_JUMP: 0x3B,
@@ -34,6 +53,9 @@ TRANSITION_BYTES = {
     TRANSITION_GRADUAL: 0x3A,
 }
 
+
+LEDNET_MUSIC_MODE_RESPONSE_LEN = 13  # 72 01 26 01 00 00 00 00 00 00 64 64 62
+LEDENET_POWER_RESTORE_RESPONSE_LEN = 7
 LEDENET_ORIGINAL_STATE_RESPONSE_LEN = 11
 LEDENET_STATE_RESPONSE_LEN = 14
 LEDENET_POWER_RESPONSE_LEN = 4
@@ -56,24 +78,36 @@ LEDENET_IC_STATE_RESPONSE_LEN = 11
 MSG_ORIGINAL_POWER_STATE = "original_power_state"
 MSG_ORIGINAL_STATE = "original_state"
 
+MSG_POWER_RESTORE_STATE = "power_restore_state"
 MSG_POWER_STATE = "power_state"
 MSG_STATE = "state"
 
+MSG_MUSIC_MODE_STATE = "music_mode_state"
 MSG_ADDRESSABLE_STATE = "addressable_state"
 
 MSG_IC_CONFIG = "ic_config"
 
-MSG_FIRST_BYTE = {
-    0xF0: MSG_POWER_STATE,
-    0x0F: MSG_POWER_STATE,
-    0x78: MSG_ORIGINAL_POWER_STATE,
-    0x66: MSG_ORIGINAL_STATE,
-    0x81: MSG_STATE,
-    0xB0: MSG_ADDRESSABLE_STATE,
-    0x00: MSG_IC_CONFIG,
+
+OUTER_MESSAGE_FIRST_BYTE = 0xB0
+
+MSG_UNIQUE_START = {
+    (0xF0, 0x71): MSG_POWER_STATE,
+    (0x0F, 0x71): MSG_POWER_STATE,
+    (0x00, 0x71): MSG_POWER_STATE,
+    (0xF0, 0x32): MSG_POWER_RESTORE_STATE,
+    (0x0F, 0x32): MSG_POWER_RESTORE_STATE,
+    (0x00, 0x32): MSG_POWER_RESTORE_STATE,
+    (0x78,): MSG_ORIGINAL_POWER_STATE,
+    (0x66,): MSG_ORIGINAL_STATE,
+    (0x81,): MSG_STATE,
+    (0x00, 0x63): MSG_IC_CONFIG,
+    (0x72,): MSG_MUSIC_MODE_STATE,
 }
+
 MSG_LENGTHS = {
+    MSG_MUSIC_MODE_STATE: LEDNET_MUSIC_MODE_RESPONSE_LEN,
     MSG_POWER_STATE: LEDENET_POWER_RESPONSE_LEN,
+    MSG_POWER_RESTORE_STATE: LEDENET_POWER_RESTORE_RESPONSE_LEN,
     MSG_ORIGINAL_POWER_STATE: LEDENET_POWER_RESPONSE_LEN,
     MSG_ORIGINAL_STATE: LEDENET_ORIGINAL_STATE_RESPONSE_LEN,
     MSG_STATE: LEDENET_STATE_RESPONSE_LEN,
@@ -81,6 +115,23 @@ MSG_LENGTHS = {
     MSG_IC_CONFIG: LEDENET_IC_STATE_RESPONSE_LEN,
 }
 
+OUTER_MESSAGE_WRAPPER = [OUTER_MESSAGE_FIRST_BYTE, 0xB1, 0xB2, 0xB3, 0x00, 0x01, 0x01]
+OUTER_MESSAGE_WRAPPER_START_LEN = 10
+CHECKSUM_LEN = 1
+
+
+POWER_RESTORE_BYTES_TO_POWER_RESTORE = {
+    restore_state.value: restore_state for restore_state in PowerRestoreState
+}
+
+
+def _message_type_from_start_of_msg(data: bytes) -> Optional[str]:
+    if len(data) > 1:
+        return MSG_UNIQUE_START.get(
+            (data[0], data[1]), MSG_UNIQUE_START.get((data[0],))
+        )
+    return MSG_UNIQUE_START.get((data[0],))
+
 
 class LEDENETOriginalRawState(NamedTuple):
     head: int
@@ -188,9 +239,23 @@ class ProtocolBase:
             self._counter = 0
         return self._counter
 
-    def is_valid_addressable_response(self, data: bytes) -> bool:
-        """Check if a message is a valid addressable state response."""
-        return False
+    def is_valid_power_restore_state_response(self, msg: bytes) -> bool:
+        """Check if a power state response is valid."""
+        return (
+            _message_type_from_start_of_msg(msg) == MSG_POWER_RESTORE_STATE
+            and len(msg) == LEDENET_POWER_RESTORE_RESPONSE_LEN
+            and self.is_checksum_correct(msg)
+        )
+
+    def is_valid_outer_message(self, data: bytes) -> bool:
+        """Check if a message is a valid outer message."""
+        if not data.startswith(bytearray(OUTER_MESSAGE_WRAPPER)):
+            return False
+        return self.is_checksum_correct(data)
+
+    def extract_inner_message(self, msg: bytes) -> bytes:
+        """Extract the inner message from a wrapped message."""
+        return msg[10:-1]
 
     def is_valid_ic_response(self, data: bytes) -> bool:
         """Check if a message is a valid ic state response."""
@@ -202,10 +267,20 @@ class ProtocolBase:
         If the response is unknown, we assume the response is
         a complete message since we have no way of knowing otherwise.
         """
-        msg_type = MSG_FIRST_BYTE.get(data[0])
+        if data[0] == OUTER_MESSAGE_FIRST_BYTE:  # This is a wrapper message
+            if len(data) < OUTER_MESSAGE_WRAPPER_START_LEN:
+                return OUTER_MESSAGE_WRAPPER_START_LEN
+            inner_msg_len = (data[8] << 8) + data[9]
+            return (
+                OUTER_MESSAGE_WRAPPER_START_LEN  # Includes the two bytes that are the size of the inner message
+                + inner_msg_len  # The inner message itself (with checksum)
+                + CHECKSUM_LEN  # The checksum of the full message
+            )
+
+        msg_type = _message_type_from_start_of_msg(data)
         if msg_type is None:
             return len(data)
-        return MSG_LENGTHS.get(msg_type, len(data))
+        return MSG_LENGTHS[msg_type]
 
     @abstractmethod
     def construct_state_query(self) -> bytearray:
@@ -248,6 +323,35 @@ class ProtocolBase:
     def construct_state_change(self, turn_on: int) -> bytearray:
         """The bytes to send for a state change request."""
 
+    def construct_power_restore_state_query(self) -> bytearray:
+        """The bytes to send for a query power restore state."""
+        return self.construct_message(bytearray([0x32, 0x3A, 0x3B, 0x0F]))
+
+    def construct_power_restore_state_change(
+        self, restore_state: PowerRestoreStates
+    ) -> bytearray:
+        """The bytes to send for a power restore state change.
+
+        Set power on state to keep last state
+        31f0f0f0f0f0e1
+        Set power on state to always on
+        310ff0f0f0f000
+        Set power on state to always off
+        31fff0f0f0f0f0
+        """
+        return self.construct_message(
+            bytearray(
+                [
+                    0x31,
+                    restore_state.channel1.value if restore_state.channel1 else 0x00,
+                    restore_state.channel2.value if restore_state.channel2 else 0x00,
+                    restore_state.channel3.value if restore_state.channel3 else 0x00,
+                    restore_state.channel4.value if restore_state.channel4 else 0x00,
+                    0xF0,
+                ]
+            )
+        )
+
     @abstractmethod
     def construct_music_mode(
         self,
@@ -321,6 +425,27 @@ class ProtocolBase:
     def construct_message(self, raw_bytes: bytearray) -> bytearray:
         """Original protocol uses no checksum."""
 
+    def construct_wrapped_message(
+        self, msg: bytearray, inner_pre_constructed: bool = False
+    ) -> bytearray:
+        """Construct a wrapped message."""
+        if inner_pre_constructed:  # msg has already been inner_pre_constructed
+            inner_msg = msg
+        else:
+            inner_msg = self.construct_message(msg)
+        inner_msg_len = len(inner_msg)
+        return self.construct_message(
+            bytearray(
+                [
+                    *OUTER_MESSAGE_WRAPPER,
+                    self._increment_counter(),
+                    inner_msg_len >> 8,
+                    inner_msg_len & 0xFF,
+                    *inner_msg,
+                ]
+            )
+        )
+
     @abstractmethod
     def named_raw_state(
         self, raw_state: bytes
@@ -397,9 +522,6 @@ class ProtocolLEDENETOriginal(ProtocolBase):
 class ProtocolLEDENET8Byte(ProtocolBase):
     """The newer LEDENET protocol with checksums that uses 8 bytes to set state."""
 
-    ADDRESSABLE_HEADER = [0xB0, 0xB1, 0xB2, 0xB3, 0x00, 0x01, 0x01]
-    addressable_response_length = MSG_LENGTHS[MSG_ADDRESSABLE_STATE]
-
     @property
     def name(self) -> str:
         """The name of the protocol."""
@@ -423,7 +545,7 @@ class ProtocolLEDENET8Byte(ProtocolBase):
 
     def _is_start_of_power_state_response(self, data: bytes) -> bool:
         """Check if a message is the start of a state response."""
-        return len(data) >= 1 and MSG_FIRST_BYTE[data[0]] == MSG_POWER_STATE
+        return _message_type_from_start_of_msg(data) == MSG_POWER_STATE
 
     def is_valid_state_response(self, raw_state: bytes) -> bool:
         """Check if a state response is valid."""
@@ -549,23 +671,9 @@ class ProtocolLEDENET8Byte(ProtocolBase):
         37 02 00 39  Jump
         37 03 00 3a  Strobe
         """
-        # Valid modes
-        # 0x00 - Fade In
+        # Valid modes for old protocol
         # 0x01 - Gradual
-        # 0x02 - Jump
-        # 0x03 - Strobe
-        return [
-            self.construct_message(bytearray([0x73, 0x01, sensitivity, 0x0F])),
-            self.construct_message(bytearray([0x37, mode or 0x00, 0x00])),
-        ]
-
-    def is_valid_addressable_response(self, data: bytes) -> bool:
-        """Check if a message is a valid addressable state response."""
-        if len(data) != self.addressable_response_length:
-            return False
-        if not data.startswith(bytearray(self.ADDRESSABLE_HEADER)):
-            return False
-        return self.is_checksum_correct(data)
+        return [self.construct_message(bytearray([0x73, 0x01, sensitivity, 0x0F]))]
 
 
 class ProtocolLEDENET8ByteAutoOn(ProtocolLEDENET8Byte):
@@ -613,6 +721,57 @@ class ProtocolLEDENET8ByteDimmableEffects(ProtocolLEDENET8ByteAutoOn):
         delay = utils.speedToDelay(speed)
         return self.construct_message(bytearray([0x38, pattern, delay, brightness]))
 
+    def construct_music_mode(
+        self,
+        sensitivity: int,
+        brightness: int,
+        mode: Optional[int],
+        effect: Optional[int],
+        foreground_colors: Optional[Tuple[int, int, int]] = None,
+        background_colors: Optional[Tuple[int, int, int]] = None,
+    ) -> List[bytearray]:
+        """The bytes to send for music mode.
+
+        Known messages
+        73 01 4d 0f d0
+              ^^
+              Likely sensitivity from 0-100 (0x64)
+        73 01 64 0f e7
+        73 01 4a 0f cd
+        73 01 4b 0f ce
+        73 01 00 0f 83
+        73 01 1b 0f 9e
+        73 01 05 0f 88
+        73 01 02 0f 85
+        73 01 06 0f 89
+        73 01 05 0f 88
+        73 01 10 0f 93
+        73 01 4d 0f d0
+        73 01 64 0f e7
+
+        Pause music mode
+        73 00 59 0f db
+           ^^
+           On/off byte
+
+        Mic
+        37 00 00 37  Fade In
+           ^^
+           Mic effect
+        37 01 00 38  Gradual
+        37 02 00 39  Jump
+        37 03 00 3a  Strobe
+        """
+        # Valid modes
+        # 0x00 - Fade In
+        # 0x01 - Gradual
+        # 0x02 - Jump
+        # 0x03 - Strobe
+        return [
+            self.construct_message(bytearray([0x73, 0x01, sensitivity, 0x0F])),
+            self.construct_message(bytearray([0x37, mode or 0x00, 0x00])),
+        ]
+
 
 class ProtocolLEDENET9Byte(ProtocolLEDENET8Byte):
     """The newer LEDENET protocol with checksums that uses 9 bytes to set state."""
@@ -920,20 +1079,9 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
         self, pattern: int, speed: int, brightness: int
     ) -> bytearray:
         """The bytes to send for a preset pattern."""
-        return self.construct_message(
-            bytearray(
-                [
-                    *self.ADDRESSABLE_HEADER,
-                    self._increment_counter(),
-                    0x00,
-                    0x05,
-                    0x42,
-                    pattern,
-                    speed,
-                    brightness,
-                    0x00,
-                ]
-            )
+        return self.construct_wrapped_message(
+            super().construct_preset_pattern(pattern, speed, brightness),
+            inner_pre_constructed=True,
         )
 
     # To query music mode
@@ -975,20 +1123,15 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
                                                                        ^^
                                                                        Likely brightness from 0-100 (0x64)
         """
-        inner_message = super().construct_music_mode(
-            sensitivity, brightness, mode, effect, foreground_colors, background_colors
-        )
         return [
-            self.construct_message(
-                bytearray(
-                    [
-                        *self.ADDRESSABLE_HEADER,
-                        self._increment_counter(),
-                        0x00,
-                        0x0D,
-                        *inner_message[0],
-                    ]
-                )
+            self.construct_wrapped_message(msg, inner_pre_constructed=True)
+            for msg in super().construct_music_mode(
+                sensitivity,
+                brightness,
+                mode,
+                effect,
+                foreground_colors,
+                background_colors,
             )
         ]
 
@@ -1043,36 +1186,11 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
         Set Red
         b0b1b2b30001010d0034a0000600010000ff0000ff0002ff00000000ff00030000ff0000ff0004ff00000000ff00050000ff0000ff0006ff00000000ffaf67
         """
-        preset_number = 0x01  # aka fixed color
-        inner_message = self.construct_message(
-            bytearray(
-                [
-                    0x41,
-                    preset_number,
-                    red,
-                    green,
-                    blue,
-                    0x00,
-                    0x00,
-                    0x00,
-                    0x06,
-                    0x01,
-                    0x00,
-                    0x00,
-                ]
-            )
-        )
-
-        return self.construct_message(
-            bytearray(
-                [
-                    *self.ADDRESSABLE_HEADER,
-                    self._increment_counter(),
-                    0x00,
-                    0x0D,
-                    *inner_message,
-                ]
-            )
+        return self.construct_wrapped_message(
+            super().construct_levels_change(
+                persist, red, green, blue, warm_white, cool_white, write_mode
+            ),
+            inner_pre_constructed=True,
         )
 
     def construct_zone_change(
@@ -1110,6 +1228,8 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
         590063ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000ff0000001e05640025
         """
         sent_zones = len(rgb_list)
+        if sent_zones > points:
+            raise ValueError(f"Device supports a maximum of {points} zones")
         pixel_bits = 9 + (points * 3)
         pixels = bytearray([pixel_bits >> 8, pixel_bits & 0xFF])
         msg = bytearray([0x59])
@@ -1118,28 +1238,15 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
         remaining = points
         for rgb in rgb_list:
             for _ in range(zone_size):
-                r, g, b = rgb
-                msg.extend(bytearray([r, g, b]))
                 remaining -= 1
+                msg.extend(bytearray([*rgb]))
         while remaining:
             remaining -= 1
-            r, g, b = rgb_list[-1]
-            msg.extend(bytearray([r, g, b]))
+            msg.extend(bytearray([*rgb]))
         msg.extend(bytearray([0x00, 0x1E]))
         msg.extend(bytearray([effect.value, speed]))
         msg.append(0x00)
-        inner_message = self.construct_message(msg)
-
-        return self.construct_message(
-            bytearray(
-                [
-                    *self.ADDRESSABLE_HEADER,
-                    self._increment_counter(),
-                    *pixels,
-                    *inner_message,
-                ]
-            )
-        )
+        return self.construct_wrapped_message(msg)
 
 
 class ProtocolLEDENETCCT(ProtocolLEDENET9Byte):
@@ -1190,7 +1297,7 @@ class ProtocolLEDENETCCT(ProtocolLEDENET9Byte):
         scaled_temp, brightness = white_levels_to_scaled_color_temp(
             warm_white, cool_white
         )
-        inner_message = self.construct_message(
+        return self.construct_wrapped_message(
             bytearray(
                 [
                     0x35,
@@ -1207,14 +1314,174 @@ class ProtocolLEDENETCCT(ProtocolLEDENET9Byte):
             )
         )
 
-        return self.construct_message(
+
+class ProtocolLEDENETAddressableChristmas(ProtocolLEDENETAddressableBase):
+    @property
+    def name(self) -> str:
+        """The name of the protocol."""
+        return PROTOCOL_LEDENET_ADDRESSABLE_CHRISTMAS
+
+    @property
+    def zones(self) -> bool:
+        """If the protocol supports zones."""
+        return True
+
+    @property
+    def power_push_updates(self) -> bool:
+        """If True the protocol pushes power state updates when controlled via ir/rf/app."""
+        return True
+
+    @property
+    def dimmable_effects(self) -> bool:
+        """Protocol supports dimmable effects."""
+        return False
+
+    @property
+    def requires_turn_on(self) -> bool:
+        """If True the device must be turned on before setting level/patterns/modes."""
+        return False
+
+    def construct_preset_pattern(
+        self, pattern: int, speed: int, brightness: int
+    ) -> bytearray:
+        """The bytes to send for a preset pattern.
+
+        OUTER_MESSAGE_WRAPPER = [0xB0, 0xB1, 0xB2, 0xB3, 0x00, 0x01, 0x01]
+
+        b0 b1 b2 b3 00 01 01 2b 00 07 a3 01 10 00 00 00 b4 62
+
+          inner = a3 01 10 00 00 00 b4
+        """
+        return self.construct_wrapped_message(
             bytearray(
                 [
-                    *self.ADDRESSABLE_HEADER,
-                    self._increment_counter(),
+                    0xA3,
+                    pattern,
+                    utils.speedToDelay(speed),
+                    0x00,
+                    0x00,
                     0x00,
-                    0x09,
-                    *inner_message,
                 ]
             )
         )
+
+    def construct_levels_change(
+        self,
+        persist: int,
+        red: int,
+        green: int,
+        blue: int,
+        warm_white: int,
+        cool_white: int,
+        write_mode: LevelWriteMode,
+    ) -> bytearray:
+        """The bytes to send for a level change request.
+
+
+        Green 100%:
+        b0b1b2b300010180000d3ba100646400000000000000a49d
+
+        Blue 50%
+        b0b1b2b300010110000d3ba176e4320000000000000068b5
+
+        Red & green 255 and 25% bright
+        b0b1b2b300010133000d3ba11e64190000000000000077f6
+
+        Red & Blue 255 and 40%
+        b0b1b2b30001014e000d3ba196642800000000000000fe1f
+
+        Inner messages
+
+        Single - Green - Brightness 100%
+        3b a1 3c 64 64 00 00 00 00 00 00 00 e0
+
+        Single - Green - Brightness 50%
+        3b a1 3c 64 32 00 00 00 00 00 00 00 ae
+
+        Single - Blue - Brightness 100%
+        3b a1 78 64 64 00 00 00 00 00 00 00 1c
+
+        Single - Red - Brightness 100%
+        3b a1 00 64 64 00 00 00 00 00 00 00 a4
+
+        Single - Pink (100% Red, 100% Blue) - Brightness 100%
+        3b a1 96 64 64 00 00 00 00 00 00 00 3a
+
+        Single - White (100% Red, 100% Green, 100% Blue) - Brightness 100%
+        3b a1 00 00 64 00 00 00 00 00 00 00 40
+
+        Single - Yellow (100% Red, 100% Green) - Brightness 100%
+        3b a1 1e 64 64 00 00 00 00 00 00 00 c2
+
+        Single - Light Blue (100% Blue, 100% Green) - Brightness 100%
+        3b a1 5a 64 64 00 00 00 00 00 00 00 fe
+
+        Single - Red - Brightness 0%
+        3b a1 00 64 00 00 00 00 00 00 00 00 40
+
+        Single - Red - Brightness 50%
+        3b a1 00 64 32 00 00 00 00 00 00 00 72
+
+        Single - Blue - Brightness 50%
+        3b a1 78 64 32 00 00 00 00 00 00 00 ea
+        """
+        h, s, v = colorsys.rgb_to_hsv(red / 255, green / 255, blue / 255)
+        return self.construct_wrapped_message(
+            bytearray(
+                [
+                    0x3B,
+                    0xA1,
+                    int(h * 180),
+                    int(s * 100),
+                    int(v * 100),
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                ]
+            )
+        )
+
+    def construct_zone_change(
+        self,
+        points: int,  # the number of points on the strip
+        rgb_list: List[Tuple[int, int, int]],
+        speed: int,
+        effect: MultiColorEffects,
+    ) -> bytearray:
+        """The bytes to send for multiple zones.
+
+        6 Zone All red
+        a000060001ff00000000ff0002ff00000000ff0003ff00000000ff0004ff00000000ff0005ff00000000ff0006ff00000000ffaf
+        6 Zone All Yellow
+        a000060001ffff000000ff0002ffff000000ff0003ffff000000ff0004ffff000000ff0005ffff000000ff0006ffff000000ffa9
+        6 Zone All Green
+        a00006000100ff000000ff000200ff000000ff000300ff000000ff000400ff000000ff000500ff000000ff000600ff000000ffaf
+        6 Zone All Green
+        a00006000100ff000000ff000200ff000000ff000300ff000000ff000400ff000000ff000500ff000000ff000600ff000000ffaf
+        6 Zone All Cyan
+        a00006000100ffff0000ff000200ffff0000ff000300ffff0000ff000400ffff0000ff000500ffff0000ff000600ffff0000ffa9
+        6 Zone All White
+        a000060001ffffff0000ff0002ffffff0000ff0003ffffff0000ff0004ffffff0000ff0005ffffff0000ff0006ffffff0000ffa3
+        """
+        sent_zones = len(rgb_list)
+        if sent_zones > points:
+            raise ValueError(f"Device supports a maximum of {points} zones")
+        msg = bytearray([0xA0, 0x00, 0x60])
+        zone_size = points // sent_zones
+        remaining = points
+        for rgb in rgb_list:
+            for _ in range(zone_size):
+                remaining -= 1
+                msg.extend(
+                    bytearray([0x00, points - remaining, *rgb, 0x00, 0x00, 0xFF])
+                )
+        while remaining:
+            remaining -= 1
+            msg.extend(
+                bytearray([0x00, points - remaining, *rgb_list[-1], 0x00, 0x00, 0xFF])
+            )
+        return self.construct_wrapped_message(msg)
